"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.stringify = exports.parse = void 0;
const base64_1 = require("../util/base64");
const isUint8Array_1 = require("../util/isUint8Array");
const constants_1 = require("./constants");
const binUriStartLength = constants_1.binUriStart.length;
const unwrapBinary = (value) => {
    if (!value)
        return value;
    if (value instanceof Array) {
        const len = value.length;
        for (let i = 0; i < len; i++) {
            const item = value[i];
            switch (typeof item) {
                case 'object': {
                    unwrapBinary(item);
                    continue;
                }
                case 'string': {
                    if (item.length < binUriStartLength)
                        continue;
                    if (item.substring(0, binUriStartLength) !== constants_1.binUriStart)
                        continue;
                    value[i] = (0, base64_1.fromBase64)(item.substring(binUriStartLength));
                }
            }
        }
        return value;
    }
    if (typeof value === 'object') {
        for (const key in value) {
            const item = value[key];
            switch (typeof item) {
                case 'object': {
                    unwrapBinary(item);
                    continue;
                }
                case 'string': {
                    if (item.length < binUriStartLength)
                        continue;
                    if (item.substring(0, binUriStartLength) !== constants_1.binUriStart)
                        continue;
                    const buf = (0, base64_1.fromBase64)(item.substring(binUriStartLength));
                    value[key] = buf;
                }
            }
        }
        return value;
    }
    if (typeof value === 'string') {
        if (value.length < binUriStartLength)
            return value;
        if (value.substring(0, binUriStartLength) !== constants_1.binUriStart)
            return value;
        return (0, base64_1.fromBase64)(value.substring(binUriStartLength));
    }
    return value;
};
const parse = (json) => {
    const parsed = JSON.parse(json);
    return unwrapBinary(parsed);
};
exports.parse = parse;
const wrapBinary = (value) => {
    if (!value)
        return value;
    if ((0, isUint8Array_1.isUint8Array)(value))
        return constants_1.binUriStart + (0, base64_1.toBase64)(value);
    if (value instanceof Array) {
        const out = [];
        const len = value.length;
        for (let i = 0; i < len; i++) {
            const item = value[i];
            out.push(!item || typeof item !== 'object' ? item : wrapBinary(item));
        }
        return out;
    }
    if (typeof value === 'object') {
        const out = {};
        for (const key in value) {
            const item = value[key];
            out[key] = !item || typeof item !== 'object' ? item : wrapBinary(item);
        }
        return out;
    }
    return value;
};
const stringify = (value, replacer, space) => {
    const wrapped = wrapBinary(value);
    return JSON.stringify(wrapped, replacer, space);
};
exports.stringify = stringify;
