"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.stringify = exports.parse = void 0;
var base64_1 = require("../util/base64");
var isUint8Array_1 = require("../util/isUint8Array");
var constants_1 = require("./constants");
var binUriStartLength = constants_1.binUriStart.length;
var unwrapBinary = function (value) {
    if (!value)
        return value;
    if (value instanceof Array) {
        var len = value.length;
        for (var i = 0; i < len; i++) {
            var item = value[i];
            switch (typeof item) {
                case 'object': {
                    unwrapBinary(item);
                    continue;
                }
                case 'string': {
                    if (item.length < binUriStartLength)
                        continue;
                    if (item.substring(0, binUriStartLength) !== constants_1.binUriStart)
                        continue;
                    value[i] = (0, base64_1.fromBase64)(item.substring(binUriStartLength));
                }
            }
        }
        return value;
    }
    if (typeof value === 'object') {
        for (var key in value) {
            var item = value[key];
            switch (typeof item) {
                case 'object': {
                    unwrapBinary(item);
                    continue;
                }
                case 'string': {
                    if (item.length < binUriStartLength)
                        continue;
                    if (item.substring(0, binUriStartLength) !== constants_1.binUriStart)
                        continue;
                    var buf = (0, base64_1.fromBase64)(item.substring(binUriStartLength));
                    value[key] = buf;
                }
            }
        }
        return value;
    }
    if (typeof value === 'string') {
        if (value.length < binUriStartLength)
            return value;
        if (value.substring(0, binUriStartLength) !== constants_1.binUriStart)
            return value;
        return (0, base64_1.fromBase64)(value.substring(binUriStartLength));
    }
    return value;
};
var parse = function (json) {
    var parsed = JSON.parse(json);
    return unwrapBinary(parsed);
};
exports.parse = parse;
var wrapBinary = function (value) {
    if (!value)
        return value;
    if ((0, isUint8Array_1.isUint8Array)(value))
        return constants_1.binUriStart + (0, base64_1.toBase64)(value);
    if (value instanceof Array) {
        var out = [];
        var len = value.length;
        for (var i = 0; i < len; i++) {
            var item = value[i];
            out.push(!item || typeof item !== 'object' ? item : wrapBinary(item));
        }
        return out;
    }
    if (typeof value === 'object') {
        var out = {};
        for (var key in value) {
            var item = value[key];
            out[key] = !item || typeof item !== 'object' ? item : wrapBinary(item);
        }
        return out;
    }
    return value;
};
var stringify = function (value, replacer, space) {
    var wrapped = wrapBinary(value);
    return JSON.stringify(wrapped, replacer, space);
};
exports.stringify = stringify;
